# How to demo the quadcopter

Follow this How-To to get the quadcopter up and running in Coover 3050. 

**Table of Contents**

[[_TOC_]]

## Setup Infrared Camera System

1. To start up the camera system, log into the camera system computer (co3050-07) with the following username and password:

    username: `.\microcart`<br>
    password: `microcart` (this might be wrong...)

2. Once the OS is done loading, start up the program "Tracking Tools".
3. From the startup window, choose "Open existing project"
4. Choose "TrackingToolsProject 2017-01-13 5.30pm" in the "Optitrack_Configuration" folder
6. Then go to File -> Open and choose "Microcart" in the "Optitrack_Configuration" folder
  - This should create a "UAV" under "Trackables" in the Project Explorer on the left side of the screen
7. Now you should be able to move the quadcopter trackable around in the tracking area, and see it update in real-time on the screen.

## Setup Ground Station
On the ground station computer (Co3050-09), log in with the following credentials. 

username: `ucart`<br>
password: `microcart`

Navigate to the ground station folder in a Terminal. 
```bash
$ cd {project_root}/groundStation
$ ls
BackEnd  Cli  logs  Makefile  obj  README.md  src  ucart.socket (or something like that)
```

If the project hasn't been built in a while, re-make the project:
```bash
make vrpn
make
```

And set the wifi environment variable if you want to connect to the quad over wifi.
```bash
$ UCART_USE_WIFI=true
```

## Setup Transmitter
The RC transmitter is used to manually control the quad.
1. Ensure the transmitter has the following state before turning it on:
  - "Gear" is set to 0
  - "Flap" is set to 1
  - Throttle is set to the lowest position
2. Turn on the transmitter.

## Setup Quadcopter

This section assumes the quad already has a prepared boot image inserted into the SD card port and that a properly charged Li-Po battery is ready for use.

0. Prequisites
  1. Make sure the connection to the motors from the main power line is **disconnected**.
  2. Make sure the previous setup sections have been done prior starting this section
1. Insert the Li-Po battery into the holder beneath the quad, and plug it into the quad.
2. Turn on the Zybo Board using the switch.
  1. The "PGOOD" Light should turn red.
  2. After the program has been completely loaded, the green DONE LED should turn on.
  ![image](/uploads/3aa1363e9f75982f259c2631bf9bcb8f/image.png)
3. Ensure the quad and transmitter has connected successfully.
  - The RC transmitter should have GAUI 330X selected and displayed on the screen. With the quad and transmitter on, the unit on the quad labeled Spektrum AR610 should have a blinking orange light or solid orange light (It is easier to see the orange light from the top of the receiver). If this is not blinking or solid, try restarting the quad and transmitter with the transmitter closer to the quad. 
  ![image](/uploads/f08672919265c5ff0017d28c1ba97770/image.png)
  ![image](/uploads/1a10172835c079f41395638d871f98e1/image.png)

4. Plug connect the motors to the main power line.

## Start the Ground Station (CLI)
Execute the following on the ground station from the root of the ground station folder.

In one terminal, run the backend:
```bash
UCART_SOCKET=./ucart.socket ./BackEnd
```

In another terminal, run the monitor:
```bash
UCART_SOCKET=./ucart.socket ./Cli monitor -f
```

Finally, in another terminal, export the socket path, and then execute any CLI commands that you like:
```bash
export UCART_SOCKET=./ucart.socket
./Cli setpid --pitch -p 1.000
# ... other CLI commands
```

## Start the Quad
1. Using the transmitter, flip the "Gear" switch to 1.
  1. You should see the green LED4 MIO7 turn on.
2. Start flying the quad. Below is a summary of how the manual controls work.
  ![image](/uploads/b9e51151c344cba8f42f6b1d21273f31/image.png)


